package livresip.uaclient;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.InetAddress;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.ArrayList;
import java.util.Properties;
import java.util.TooManyListenersException;

import javax.sip.ClientTransaction;
import javax.sip.Dialog;
import javax.sip.DialogTerminatedEvent;
import javax.sip.IOExceptionEvent;
import javax.sip.InvalidArgumentException;
import javax.sip.ListeningPoint;
import javax.sip.PeerUnavailableException;
import javax.sip.RequestEvent;
import javax.sip.ResponseEvent;
import javax.sip.SipException;
import javax.sip.SipFactory;
import javax.sip.SipListener;
import javax.sip.SipProvider;
import javax.sip.SipStack;
import javax.sip.TimeoutEvent;
import javax.sip.Transaction;
import javax.sip.TransactionTerminatedEvent;
import javax.sip.address.Address;
import javax.sip.address.AddressFactory;
import javax.sip.address.SipURI;
import javax.sip.header.CSeqHeader;
import javax.sip.header.CallIdHeader;
import javax.sip.header.ContactHeader;
import javax.sip.header.ContentTypeHeader;
import javax.sip.header.FromHeader;
import javax.sip.header.HeaderFactory;
import javax.sip.header.MaxForwardsHeader;
import javax.sip.header.ToHeader;
import javax.sip.header.ViaHeader;
import javax.sip.message.MessageFactory;
import javax.sip.message.Request;
import javax.sip.message.Response;

import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;


/**
 * 	Simple SIP User Agent Client 
 * 
 *  @see Chapitre 4 du livre Tlphonie SIP : concepts, usages et programmation en Java
 *	@author G. Vanwormhoudt
 */

public class UserAgentClient implements SipListener {

	protected static Logger logger = Logger.getLogger(UserAgentClient.class);
	
	protected static SipFactory sipFactory;
	protected static AddressFactory addressFactory;
	protected static MessageFactory messageFactory;
	protected static HeaderFactory headerFactory;
	protected static SipStack sipStack;
	
	protected ListeningPoint listeningPoint;
	protected SipProvider sipProvider;

	protected long cSeq;
	
	protected Dialog dialog;

	protected int dataPort;
	protected DayTimePlayer player; 
	
	protected int cnt; 
	
	public UserAgentClient(int port, String transport, int dport) 
	 	throws SipException, TooManyListenersException, InvalidArgumentException  {
	
		cSeq = 1;
		cnt = 0;
		dataPort = dport; 
					
		listeningPoint = sipStack.createListeningPoint(sipStack.getIPAddress(), port, transport);
		sipProvider = sipStack.createSipProvider(listeningPoint);
		sipProvider.addSipListener(this); 
		
	}
 
	public static void initSip(String host, String proxyIP) throws PeerUnavailableException  {		
		// Choix de la pile
		sipFactory = SipFactory.getInstance();
		sipFactory.setPathName("gov.nist");

		// Creation des fabriques
		headerFactory = sipFactory.createHeaderFactory();
		addressFactory = sipFactory.createAddressFactory();
		messageFactory = sipFactory.createMessageFactory();
		
		Properties properties = new Properties();
		properties.setProperty("javax.sip.STACK_NAME", "UserAgentClient");	
		properties.setProperty("javax.sip.IP_ADDRESS", host);
		properties.setProperty("javax.sip.OUTBOUND_PROXY", proxyIP);
		properties.setProperty("javax.sip.RETRANSMISSION_FILTER", "true");
		properties.setProperty("gov.nist.javax.sip.TRACE_LEVEL", "16");
		
		// Creation de la pile SIP
		sipStack = sipFactory.createSipStack(properties);	
	}

	
	/*
	 * Send an Invite Request 
	 */
	public void sendInvite() {
		
			String fromName = "gilles";
			String fromSipAddress = "here.com";
			String fromDisplayName = "The Master Blaster";

			String toSipAddress = "daytime";
			String toUser = "service.com";

			try {
				// Cration du champ d'entte From
				SipURI fromAddress =
					addressFactory.createSipURI(fromName, fromSipAddress);

				Address fromNameAddress = addressFactory.createAddress(fromAddress);
				fromNameAddress.setDisplayName(fromDisplayName);
				FromHeader fromHeader =
					headerFactory.createFromHeader(fromNameAddress, "12345");
	
				// Cration du champ d'entte To
				SipURI toAddress =
					addressFactory.createSipURI(toUser, toSipAddress);
				Address toNameAddress = addressFactory.createAddress(toAddress);
				ToHeader toHeader =
					headerFactory.createToHeader(toNameAddress, null);
	
				// Cration de l'URI de la requte
				SipURI requestURI =
					addressFactory.createSipURI(toUser, toSipAddress);
	
				// Cration du champ ViaHeaders
				ArrayList viaHeaders = new ArrayList();
				ViaHeader viaHeader =
					headerFactory.createViaHeader(
						sipStack.getIPAddress(),
						listeningPoint.getPort(),
						listeningPoint.getTransport(),
						null);
				viaHeaders.add(viaHeader);
	
				// Creation du champ d'entte CallId 
				CallIdHeader callIdHeader = sipProvider.getNewCallId();
	
				// Create du champ d'entte CSe
				CSeqHeader cSeqHeader = headerFactory.createCSeqHeader(cSeq++, Request.INVITE);
				
				// Create a new MaxForwardsHeader
				MaxForwardsHeader maxForwards =
					headerFactory.createMaxForwardsHeader(70);
	
				// Create the request.
				Request request =
					messageFactory.createRequest(
						requestURI,
						Request.INVITE,
						callIdHeader,
						cSeqHeader,
						fromHeader,
						toHeader,
						viaHeaders,
						maxForwards);

				// Create contact headers
				String host = sipStack.getIPAddress();
				SipURI contactURI = addressFactory.createSipURI(fromName, host);
				contactURI.setPort(listeningPoint.getPort());
	
				Address contactAddress = addressFactory.createAddress(contactURI);
				ContactHeader contactHeader =
					headerFactory.createContactHeader(contactAddress);
				request.addHeader(contactHeader);

				// Cration du champ d'entte Content-Type 
				ContentTypeHeader contentTypeHeader =
					headerFactory.createContentTypeHeader("text", "txt");
				String data =
					"ip=" + host + "\r\n" + 
					"port=" + dataPort + "\r\n" +
					"frequency=500" + "\r\n" + 
					"duration=10000\r\n";
	
				byte[]  contents = data.getBytes();
				request.setContent(contents, contentTypeHeader);
	
				// Envoi de la requte
				ClientTransaction inviteTransaction = sipProvider.getNewClientTransaction(request);
				inviteTransaction.sendRequest();
				
				printState(inviteTransaction);
			
			}
			catch(Exception ex) {
				ex.printStackTrace();
			}
		
	}
	
	public void sendBye() {
		try {
			if (dialog != null) {
				Request byeRequest = dialog.createRequest(Request.BYE);
				ClientTransaction byeTransaction = sipProvider.getNewClientTransaction(byeRequest);
				dialog.sendRequest(byeTransaction);
			}
		} catch (SipException e) {
			e.printStackTrace();
		} 
	}
	
	public void processRequest(RequestEvent requestReceivedEvent) {
		Request request = requestReceivedEvent.getRequest();
		logger.warn("Request received : " + request.getMethod() + "-- ignore");
	}


	public void processResponse(ResponseEvent responseReceivedEvent) {
		Response response = (Response) responseReceivedEvent.getResponse();
		ClientTransaction clientTransaction = responseReceivedEvent.getClientTransaction();
		
		if (clientTransaction == null) {
			logger.trace("Unmatched Response:" + response.getReasonPhrase() + "-- ignore ");
			return;
		}
		else {
			printState(clientTransaction);
		}
		
		try {
			// Response OK 
			if (response.getStatusCode() == Response.OK) {
				CSeqHeader  cSeqHeader = (CSeqHeader) response.getHeader(CSeqHeader.NAME);
			
				// OK pour une requte INVITE
				if (cSeqHeader.getMethod().equals(Request.INVITE)) {
					dialog = clientTransaction.getDialog();				
					Request ackRequest = dialog.createAck(cSeqHeader.getSeqNumber());
					dialog.sendAck(ackRequest);
					
					// dmarrage du lecteur
					player = new DayTimePlayer(dataPort);
					new Thread(player).start();
				} 
				else {
					// OK pour une requte BYE
					if (cSeqHeader.getMethod().equals(Request.BYE)) {
						shutdown();
					}
				} 	
			} else {
				// on ignore les autres REPONSES
				logger.trace("Unhandled Response:" + response.getReasonPhrase());
			}
			
		} catch (Exception ex) {
			ex.printStackTrace();
		}

	}
	
	public void processTimeout(TimeoutEvent event) {
		Transaction transaction = (event.isServerTransaction()) 
			? (Transaction) event.getServerTransaction() : event.getClientTransaction();
		logger.trace("Transaction Time out:" + transaction);
	}

	public void processDialogTerminated(DialogTerminatedEvent event) {
		Dialog dialog = event.getDialog();
		logger.trace("DialogTerminated:" + dialog);
	}

	public void processIOException(IOExceptionEvent event) {
		Object source = event.getSource();
		logger.trace("IOException from source:" + source);
	}

	public void processTransactionTerminated(TransactionTerminatedEvent event) {
		Transaction transaction = event.isServerTransaction() 
			? (Transaction) event.getServerTransaction() : event.getClientTransaction();
		logger.trace("Transaction Time out:" + transaction);
	}
	

	protected void printState(Transaction ct) {
		logger.trace("---" + cnt++);
		logger.trace("Transaction state is " + ct.getState());
		logger.trace("Dialog = " + ct.getDialog());
		logger.trace("Dialog State is " + ct.getDialog().getState());
		
	}

	protected void shutdown() {
		try {
			sipProvider.removeSipListener(this);
			sipStack.deleteListeningPoint(listeningPoint);
			sipStack.deleteSipProvider(sipProvider);
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}
	
	class DayTimePlayer implements Runnable {
		protected int port;
		
		DayTimePlayer(int p) { port = p; }
				
		public void run() {
			try {
				ServerSocket sockSvr = new ServerSocket( port );
				System.out.println( "Service DayTime active sur port:" + port );
				Socket socket = sockSvr.accept();				
				BufferedReader in = new BufferedReader(
		                        new InputStreamReader(
		                            socket.getInputStream()));
				String inputLine;

				while ((inputLine = in.readLine()) != null) {	
					if (inputLine.equals("STOP"))
						break;
					else {
						System.out.println("Received date: " + inputLine);
					}
				}
				socket.close();
				sendBye();
			}
			catch (IOException e) {
				e.printStackTrace();
			}
			
		}

		
	}
	
	public static void main(String args[]) throws Exception {
		Properties logProperties = new Properties();
		logProperties.put("log4j.rootLogger", "TRACE,UAC");
		logProperties.put("log4j.appender.UAC", "org.apache.log4j.ConsoleAppender");  // Our custom appender
		logProperties.put("log4j.appender.UAC.layout", "org.apache.log4j.PatternLayout"); //See: http://logging.apache.org/log4j/docs/api/org/apache/log4j/PatternLayout.html
		logProperties.put("log4j.appender.UAC.layout.ConversionPattern", "%d{HH:mm:ss} %5.5p %40.40c: %m%n");		
		PropertyConfigurator.configure(logProperties);
		
		String localIP = InetAddress.getLocalHost().getHostAddress();
		int localPort = 5060;
		int dataPort = 6022;
		int proxyPort = 5070;
		String transport = "udp"; 
		String proxy = localIP + ":" + proxyPort + "/" + transport;
		
		UserAgentClient.initSip(localIP, proxy);
		
		UserAgentClient uac = new UserAgentClient(localPort, transport, dataPort);
		uac.sendInvite();

	}

	
}

