package livresip.chat;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.net.InetAddress;
 
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.ScrollPaneConstants;
import javax.swing.SwingUtilities;

/**
 * 	Classe implantant l'interface utilisateur de la messagerie conversationnelle  
 * 
 *  @see Chapitre 6 du livre Tlphonie SIP : concepts, usages et programmation en Java
 *	@author G. Vanwormhoudt
 */

public class ChatUI 
	extends JFrame
	implements ChatSipEventListener
{
    protected ChatSip sipLayer;
    
	protected boolean isCaller; 
    
    protected JTextField fromAddress;
    protected JLabel fromLbl;
    protected JLabel receivedLbl;
    protected JTextArea receivedMessages;
    protected JScrollPane receivedScrollPane;
    protected JButton sendBtn;
    protected JLabel sendLbl;
    protected JTextField sendMessages;
    protected JTextField toAddress;
    protected JLabel toLbl;
	protected JButton connectBtn;
	protected JButton disconnectBtn;
	
    
    public ChatUI(ChatSip sip)
    {
        super();
        sipLayer = sip;
        initWindow();
        String from = "sip:" + sip.getUsername() + "@" + sip.getHost() + ":" + sip.getPort();
        this.fromAddress.setText(from);
    }
    
    protected void initWindow() {
        receivedLbl = new JLabel();
        sendLbl = new JLabel();
        sendMessages = new JTextField();
        receivedScrollPane = new JScrollPane();
        receivedMessages = new JTextArea();
        fromLbl = new JLabel();
        fromAddress = new JTextField();
        toLbl = new JLabel();
        toAddress = new JTextField();
        sendBtn = new JButton();
        connectBtn = new JButton();
        disconnectBtn = new JButton();

        getContentPane().setLayout(null);

        setTitle("TextClient");
        addWindowListener(new WindowAdapter() {
            @Override
			public void windowClosing(WindowEvent evt) {
                System.exit(0);
            }
        });

        receivedLbl.setText("Received Messages:");
        receivedLbl.setAlignmentY(0.0F);
        receivedLbl.setPreferredSize(new java.awt.Dimension(25, 100));
        getContentPane().add(receivedLbl);
        receivedLbl.setBounds(5, 0, 136, 20);

        sendLbl.setText("Send Message:");
        getContentPane().add(sendLbl);
        sendLbl.setBounds(5, 150, 90, 20);

        getContentPane().add(sendMessages);
        sendMessages.setBounds(5, 170, 270, 20);

        receivedMessages.setAlignmentX(0.0F);
        receivedMessages.setEditable(false);
        receivedMessages.setLineWrap(true);
        receivedMessages.setWrapStyleWord(true);
        receivedScrollPane.setViewportView(receivedMessages);
        receivedScrollPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);

        getContentPane().add(receivedScrollPane);
        receivedScrollPane.setBounds(5, 20, 270, 130);

        sendBtn.setText("Send");
        sendBtn.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                sendBtnActionPerformed(evt);
            }
        });
        sendBtn.setEnabled(false);
                
        getContentPane().add(sendBtn);
        sendBtn.setBounds(5, 200, 75, 25);
        
        fromLbl.setText("From:");
        getContentPane().add(fromLbl);
        fromLbl.setBounds(5, 230, 35, 15);

        getContentPane().add(fromAddress);
        fromAddress.setBounds(40, 230, 235, 20);
        fromAddress.setEditable(false);

        toLbl.setText("to:");
        getContentPane().add(toLbl);
        toLbl.setBounds(5, 250, 35, 15);
        
        getContentPane().add(toAddress);
        toAddress.setBounds(40, 250, 235, 21);

        connectBtn.setText("Connect");
        connectBtn.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                connectBtnActionPerformed(evt);
            }
        });
        getContentPane().add(connectBtn);
        connectBtn.setBounds(5, 280, 100, 25);
        
        
        disconnectBtn.setText("Disconnect");
        disconnectBtn.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                disconnectBtnActionPerformed(evt);
            }
        });
        getContentPane().add(disconnectBtn);
        disconnectBtn.setBounds(100, 280, 140, 25);
        disconnectBtn.setEnabled(false);
        
        java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        setBounds((screenSize.width-320)/2, (screenSize.height-360)/2, 320, 360);
    }

    protected void connectBtnActionPerformed(ActionEvent evt) {
    	
    	try {
    		String to = this.toAddress.getText();
    		if (! to.equals("")) {
    			this.toAddress.setEditable(false);
    			this.connectBtn.setEnabled(false);
    			this.disconnectBtn.setEnabled(true);
    			sipLayer.sendInvite(to);    
    		} else {
    			this.displayMsg("To: field can not be empty !");
    		}
		} catch (Exception e) {
			e.printStackTrace();
		} 
	}

	protected void disconnectBtnActionPerformed(ActionEvent evt) {
		try {
			sipLayer.sendBye();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	protected void sendBtnActionPerformed(ActionEvent evt) {

        try
        {
            String message = this.sendMessages.getText();
            if (! message.equals("")) {
            	sipLayer.sendMessage(message);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }			
    }
	
	public void updateUISessionEstablished(String sender) {
		final String s = sender;
		Runnable doWorkRunnable = new Runnable() {
		    public void run() { 
		    	ChatUI outer = ChatUI.this;
		    	outer.toAddress.setText("");
		    	outer.toAddress.setText(s);
				outer.toAddress.setEditable(false);
				outer.connectBtn.setEnabled(false);
				outer.disconnectBtn.setEnabled(true);
				outer.sendBtn.setEnabled(true);
				outer.displayMsg("Session established");
		    }
		};
		SwingUtilities.invokeLater(doWorkRunnable);
		
	}
	
	public void updateUISessionRequested(AcceptDeclineData data) {
		final AcceptDeclineData d = data; 
		Runnable doWorkRunnable = new Runnable() {
		    public void run() { 
		    	int option = JOptionPane.showConfirmDialog(
						ChatUI.this,
						"Accept Invitation from " + d.getInfo(), 
						"", 
						JOptionPane.YES_NO_OPTION);
		    	if (option == JOptionPane.YES_OPTION) {
		    		d.accept();
		    	} else {
		    		d.decline();
		    	}
		    }
		};
		try { SwingUtilities.invokeAndWait(doWorkRunnable); } catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	public void updateUISubscriptionRequested(AcceptDeclineData data) {
		final AcceptDeclineData d = data; 
		Runnable doWorkRunnable = new Runnable() {
		    public void run() { 
		    	int option = JOptionPane.showConfirmDialog(
						ChatUI.this,
						"Subscribe" + d.getInfo(), 
						"", 
						JOptionPane.YES_NO_OPTION);
		    	if (option == JOptionPane.YES_OPTION) {
		    		d.accept();
		    	} else {
		    		d.decline();
		    	}
		    }
		};
		try { SwingUtilities.invokeAndWait(doWorkRunnable); } catch (Exception e) {
			e.printStackTrace();
		}
		/* if (SwingUtilities.isEventDispatchThread()) {
		    // set of code here
		} else {
		    SwingUtilities.invokeAndWait(new Runnable() {
		        @Override
		        public void run() {
		            // set of code here
		        }
		    });
		} */
	}
	
	public void updateUISessionClosed() {
		Runnable doWorkRunnable = new Runnable() {
		    public void run() { 
		    	ChatUI outer = ChatUI.this;
		    	outer.toAddress.setText("");
		    	outer.toAddress.setEditable(true);
		    	outer.connectBtn.setEnabled(true);
		    	outer.disconnectBtn.setEnabled(false);
		    	outer.sendBtn.setEnabled(false);
		    	outer.displayMsg("Session closed");
		    }
		};
		SwingUtilities.invokeLater(doWorkRunnable);
		
	}
	
	public void displayMsg(String msg) {
		final String m = msg; 
		Runnable doWorkRunnable = new Runnable() {
		    public void run() { 
		    	ChatUI outer = ChatUI.this;
		    	outer.receivedMessages.append("--" + m + "\n");   	
		    }
		};
		SwingUtilities.invokeLater(doWorkRunnable);	 		
	}


	public void notify(EventType event, Object data) {
		String msg =""; 
		
		switch (event) {
		case SESSION_REQUESTED : 
				updateUISessionRequested((AcceptDeclineData) data); break;
		case SESSION_REQUESTED_ONBUSY: 
				displayMsg("Other peer : " + data  + " is trying to contact"); 
				break;
		case SESSION_CLOSED : 
				updateUISessionClosed(); break;
		case SESSION_DECLINED : 
					displayMsg("Other peer has declined the invitation"); 
					updateUISessionClosed();
					break; 
		case SESSION_ESTABLISHED: updateUISessionEstablished(data.toString()); break;
		case SESSION_BUSY : 
					displayMsg("Contacted peer is busy"); 
					updateUISubscriptionRequested((AcceptDeclineData) data); 			
					break;
		case MESSAGE_RECEIVED : displayMsg("Received Message :" + data.toString()); break;
		case MESSAGE_SENT : displayMsg("Message sent"); break;
		case SUBSCRIPTION_REQUESTED: 
				displayMsg("Subscription Requested");
				break;
		case SUBSCRIPTION_ACCEPTED: 
				displayMsg("Subscription Accepted");
				break;
		case ERROR : displayMsg("Error " + data.toString()); break;
		}
		
	}
	
	public static void main(String[] args)
    {
        if(args.length != 2) {
            printUsage();
            System.exit(-1);            
        }
        
		try
        {
		    String username = args[0];
		    int port = Integer.parseInt(args[1]);
		    String ip = InetAddress.getLocalHost().getHostAddress();
		    
			ChatSip sipLayer = new ChatSip(username, ip, port);
		    ChatUI ui = new ChatUI(sipLayer);
		    sipLayer.setChatListener(ui);
			
			ui.setVisible(true);
        } catch (Exception e)
        {
            System.out.println("Problem initializing the SIP stack.");
            e.printStackTrace();
            System.exit(-1);
        }
    }

    protected static void printUsage()
    {
    	System.out.println("Syntaxe:");
    	System.out.println("  java -jar chat.jar <username> <port>");
    	System.out.println("<username> est le pseudo de l'utilisateur");
    	System.out.println("<port> est le numro de port  utiliser");
    	System.out.println("Exemple:");
    	System.out.println("  java -jar chat.jar gva 5061");
    }
	
}
