package livresip.sipphone.media;

import java.io.File;

import javax.media.ControllerEvent;
import javax.media.ControllerListener;
import javax.media.EndOfMediaEvent;
import javax.media.Manager;
import javax.media.MediaLocator;
import javax.media.Player;
import javax.media.Time;


/**
 * 	Classe pour manipuler des sonneries contenues dans des fichiers
 * 
 *  @see Chapitre 7 du livre Tlphonie SIP : concepts, usages et programmation en Java
 *	@author G. Vanwormhoudt
 */

public class Tone implements ControllerListener {

	protected Player player = null;
	protected boolean looping = false;
	protected boolean stoping = false;
	
	/**
	 * @param path Chemin du fichier, peut tre relatif.
	 * @param loop spcifie si la sonnerie doit boucler.
	 */
	protected Tone(String path, boolean loop) {
		looping = loop;

		try {
		
			// Rcupration du chemin absolu et construction du MediaLocator
			String absPath = (new File(path).getAbsolutePath());
			MediaLocator locator = new MediaLocator("file://"+ absPath);

			// construction et initialisation du player
			player = Manager.createPlayer(locator);
			player.addControllerListener(this);
			player.realize();

			// boucle bloquante jusque ralisation du player
			while (player.getState() < Player.Realized) { 
				try { Thread.sleep(100); } 
				catch(InterruptedException ex) { }
			}
		
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}
	

	public void play() {
		stoping = false;
		player.setMediaTime(new Time(0));
		player.start();
	}
	
	public void stop() {
		stoping = true;
		player.stop();
	}
	
	public void close() {
		player.close();
	}
	
	public void controllerUpdate(ControllerEvent event) {

		// rcuperation du player
		Player p = (Player) event.getSourceController();
		if (p == player) {
			// gestion de la boucle
			if (event instanceof EndOfMediaEvent) {
				// on relance la lecture si il ne faut pas s'arrter.
				if (looping && !stoping) {
					play();
				} else if (stoping) {
					stoping = false;
				}
			}
		}
	}	
}
