package livresip.sipphone.rtp;


import javax.media.ControllerErrorEvent;
import javax.media.ControllerEvent;
import javax.media.ControllerListener;
import javax.media.Manager;
import javax.media.Player;
import javax.media.RealizeCompleteEvent;
import javax.media.protocol.DataSource;
import javax.media.rtp.RTPManager;
import javax.media.rtp.ReceiveStream;
import javax.media.rtp.ReceiveStreamListener;
import javax.media.rtp.event.ByeEvent;
import javax.media.rtp.event.NewReceiveStreamEvent;
import javax.media.rtp.event.ReceiveStreamEvent;
import javax.media.rtp.event.RemotePayloadChangeEvent;

import org.apache.log4j.Logger;

/**
 * Classe grant la rception du flux RTP
 * 
 *  @see Chapitre 7 du livre Tlphonie SIP : concepts, usages et programmation en Java
 *	@author G. Vanwormhoudt
 */


public class AudioReceiver implements ReceiveStreamListener, ControllerListener {
	protected RTPManager rtpManager = null;
	protected Player player = null;
	protected Logger logger = null;
	
	/**
	 * @param mgr manager est utilis pour rceptionner le flux
	 */
	public AudioReceiver(RTPManager mgr, Logger lgr) {
		rtpManager = mgr;
		logger = lgr; 
		// enregistrement pour traiter les vnement ReceiveStreamEvent du manager
		rtpManager.addReceiveStreamListener(this);
	}
	
	public void close() {
		if (player != null)
			player.close();
	}

	@Override
	public synchronized void update(ReceiveStreamEvent evt) {

		ReceiveStream stream = evt.getReceiveStream();

		if (evt instanceof RemotePayloadChangeEvent) {

			logger.info("Received an RTP PayloadChangeEvent.");
			logger.info("Sorry, cannot handle payload change.");
			System.exit(0);

		}

		// Rception d'un nouveau flux
		else if (evt instanceof NewReceiveStreamEvent) {
			try {
				logger.info("Receveid new RTP stream");

				// rcupration de la datasource associe
				stream = ((NewReceiveStreamEvent) evt).getReceiveStream();
				DataSource ds = stream.getDataSource();
				
				// cration du player en passant la datasource du nouveau flux
				player = Manager.createPlayer(ds);
				if (player == null)
					return;

				// on s'enregistre auprs du player pour traiter les venement puis
				// on demande la ralisation du player
				player.addControllerListener(this);
				player.realize();

			} catch (Exception e) {
				logger.debug("NewReceiveStreamEvent exception " + e.getMessage());
				return;
			}

		}
		// fin de session RTP
		else if (evt instanceof ByeEvent) {
			logger.info("RTP Session ended");
			close();
		}
	}

	@Override
	public synchronized void controllerUpdate(ControllerEvent ce) {
		
		Player p = (Player) ce.getSourceController();

		if (p == null)
			return;

		// Le player est pass  l'tat Realized, on lance la lecture
		if (ce instanceof RealizeCompleteEvent) {
			p.start();
		}

		// Gestion des erreurs
		if (ce instanceof ControllerErrorEvent) {
			p.removeControllerListener(this);
			close();
			logger.info("AudioReceiver internal error: " + ce);
		}
	}

} 

