package livresip.sipphone.sip;
import java.net.InetAddress;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Properties;
import java.util.Random;
import java.util.Timer;
import java.util.TimerTask;
import java.util.TooManyListenersException;
import java.util.Vector;

import javax.media.rtp.RTPManager;
import javax.media.rtp.SessionAddress;
import javax.sdp.Connection;
import javax.sdp.Field;
import javax.sdp.Media;
import javax.sdp.MediaDescription;
import javax.sdp.Origin;
import javax.sdp.SdpException;
import javax.sdp.SdpFactory;
import javax.sdp.SessionDescription;
import javax.sdp.SessionName;
import javax.sip.ClientTransaction;
import javax.sip.Dialog;
import javax.sip.DialogTerminatedEvent;
import javax.sip.IOExceptionEvent;
import javax.sip.InvalidArgumentException;
import javax.sip.ListeningPoint;
import javax.sip.ObjectInUseException;
import javax.sip.PeerUnavailableException;
import javax.sip.RequestEvent;
import javax.sip.ResponseEvent;
import javax.sip.ServerTransaction;
import javax.sip.SipException;
import javax.sip.SipFactory;
import javax.sip.SipListener;
import javax.sip.SipProvider;
import javax.sip.SipStack;
import javax.sip.TimeoutEvent;
import javax.sip.Transaction;
import javax.sip.TransactionTerminatedEvent;
import javax.sip.TransportNotSupportedException;
import javax.sip.address.Address;
import javax.sip.address.AddressFactory;
import javax.sip.address.SipURI;
import javax.sip.address.URI;
import javax.sip.header.AllowHeader;
import javax.sip.header.CSeqHeader;
import javax.sip.header.CallIdHeader;
import javax.sip.header.ContactHeader;
import javax.sip.header.ContentTypeHeader;
import javax.sip.header.FromHeader;
import javax.sip.header.HeaderFactory;
import javax.sip.header.MaxForwardsHeader;
import javax.sip.header.RouteHeader;
import javax.sip.header.ToHeader;
import javax.sip.header.ViaHeader;
import javax.sip.message.MessageFactory;
import javax.sip.message.Request;
import javax.sip.message.Response;

import org.apache.log4j.Logger;

import livresip.sipphone.events.BusyCallEvent;
import livresip.sipphone.events.CancelledCallEvent;
import livresip.sipphone.events.ConnectedEvent;
import livresip.sipphone.events.DeclinedEvent;
import livresip.sipphone.events.DisconnectedEvent;
import livresip.sipphone.events.MovedEvent;
import livresip.sipphone.events.NewCallEvent;
import livresip.sipphone.events.RegisteredEvent;
import livresip.sipphone.events.RingingEvent;
import livresip.sipphone.events.TemporarilyUnavailableEvent;
import livresip.sipphone.events.UnregisteredEvent;
import livresip.sipphone.events.UserAgentEvent;
import livresip.sipphone.rtp.AudioReceiver;
import livresip.sipphone.rtp.AudioTransmiter;

/**
 * 	Classe implantant la couche SIP du sipphone  
 * 
 *  @see Chapitre 7 du livre Tlphonie SIP : concepts, usages et programmation en Java
 *	@author G. Vanwormhoudt
 */

public class UserAgentSIP implements SipListener {
	
	
	protected static SipFactory sipFactory;
	protected static AddressFactory addressFactory;
	protected static MessageFactory messageFactory;
	protected static HeaderFactory headerFactory;
	protected static SdpFactory sdpFactory;
	protected SipStack sipStack;
	protected ListeningPoint listeningPoint;
	protected SipProvider sipProvider;
	
	protected Dialog currentDialog = null;
	protected ClientTransaction currentClientTransaction;
	protected ServerTransaction currentServerTransaction;
	protected long cSeq = 0;
	//Permet de savoir si l'utilisateur  un appel en cours
	
	protected Map methods = new HashMap();
	
	protected static Random localTagGenerator = new Random();
	protected static AllowHeader allowHeader;
	protected Timer registerTimer;
	
	protected Timer ringingTimer;
	protected static final long RINGING_DELAY = 2000;
	protected static final long RINGING_START = 100;
	
	// myLogicalAddress sert pour remplir le champ From du message SIP
	// myPhysicalAddress pour le champ Contact
	protected Address myLogicalAddress;
	protected Address myPhysicalAddress; 
	protected Address proxyAddress;
	protected String localName, screenName;
	//variable servant a crer l'adresse de destination lorsque l'on passe par un proxy
	protected String domain = null; 
	private String mail = null; 
	
	protected RTPManager rtpManager;
	protected AudioTransmiter transmiter;
	protected AudioReceiver receiver;
	protected int rtpPort = 22222;
	
	protected UserAgentListener listener;
	
	protected Logger logger;

	
	/**
	 * Constructeur d'un UserAgent client/serveur
	 * @param host Adresse IP de l'ordinateur hote
	 * @param port Port d'coute SIP (par dfaut 5060)
	 * @param transport Type de transport pour la signalisation (udp ou tcp)
	 * @param userID Identifiant pour les serveurs proxy ou PABX IP
	 * @param dmn Nom du domain
	 * @param mail Mail de l'utilisateur pour l'enregistrement
	 * @param proxy Adresse IP du proxy 
	 * @param log Logger de trace
	 * @throws TooManyListenersException
	 * @throws PeerUnavailableException
	 * @throws TransportNotSupportedException
	 * @throws InvalidArgumentException
	 * @throws ObjectInUseException
	 * @throws ParseException
	 */
	public UserAgentSIP(String host, int port, String transport, String userID, String dmn, String ml, String proxy, Logger log) throws
			TooManyListenersException,
			PeerUnavailableException,
			TransportNotSupportedException,
			InvalidArgumentException,
			ObjectInUseException, ParseException {
		
		// Cration des diffrentes fabriques si elles n'existent pas dj car ce sont des objets statiques 
		if (sipFactory==null) {
			initFactories();
		}

		// Cration de la pile SIP
        Properties stackProperties = new Properties();
        stackProperties.setProperty("javax.sip.STACK_NAME", "UserAgentSip");
		stackProperties.setProperty("javax.sip.IP_ADDRESS", host);
        
        //Si le champ proxy a t rempli, on s'interrompt sinon on l'applique  la proprit de la pile sip OUTBOUND_PROXY
		if (proxy == null || proxy.length() == 0) {
			throw new InvalidArgumentException("outboundProxyURI must not be null"); 
		}
		stackProperties.setProperty("javax.sip.OUTBOUND_PROXY", proxy);
		stackProperties.setProperty("gov.nist.javax.sip.TRACE_LEVEL", "16");
    
        //Cration de la pile
        sipStack = sipFactory.createSipStack(stackProperties);

        // Cration du ListeningPoint et association avec le SipProvider		
		listeningPoint  = sipStack.createListeningPoint(host, port, transport);
		sipProvider = sipStack.createSipProvider(listeningPoint);
        sipProvider.addSipListener(this);
        
        //Si un identifiant a t entr on l'utilise, sinon on utilise le nom de l'ordinateur
        screenName = System.getProperty("user.name");
        if (userID != null && userID.length() >0) {
        		localName = userID;
        }
        else {
        	localName = screenName.toLowerCase().replaceAll(" ", "");
        }
        
        proxyAddress = addressFactory.createAddress("sip:" + proxy);
        
        //Cration de l'adresse qui sera affiche sur l'cran de l'appel
		String physicalURI = listeningPoint.getIPAddress()+":"+listeningPoint.getPort();
		SipURI physicalSipURI = addressFactory.createSipURI(localName, physicalURI);
		myPhysicalAddress = addressFactory.createAddress(screenName, physicalSipURI);
		
		
		//Ajout du proxyAddress, s'il en existe un, pour fabriquer l'en-tte SIP From
		domain = dmn;
		SipURI logicalSipURI = addressFactory.createSipURI(localName, domain);
		myLogicalAddress = addressFactory.createAddress(screenName, logicalSipURI);
		
		mail = ml;
	
		logger = log; 
		
		logger.info("User agent: " + myLogicalAddress);
        logger.info("Now listening at " + listeningPoint.getIPAddress() + ":" + listeningPoint.getPort() + "/" + listeningPoint.getTransport());
	}

	
	//Cration de toutes les fabriques ncessaires pour formater des messages SIP
	protected void initFactories() throws PeerUnavailableException, ParseException{
        sipFactory = SipFactory.getInstance();
        sipFactory.setPathName("gov.nist");
        headerFactory = sipFactory.createHeaderFactory();
        addressFactory = sipFactory.createAddressFactory();
        messageFactory = sipFactory.createMessageFactory(); 
        sdpFactory = SdpFactory.getInstance();
        allowHeader = headerFactory.createAllowHeader("INVITE, ACK, BYE, OPTIONS, CANCEL");
	}

	
	public boolean isValidSipAddress(String address) {
		boolean result = false;
		try {
			addressFactory.createURI("sip:" + address);
			result = true; 
		} 
		catch(ParseException pe) {	}
		return result;
	}
	
	
	/**
	 * Stop le UserAgent en arrtant proprement la pile SIP et le RTPManager
	 *
	 */
	 public void stop() {
		try {
			destroyRTP();
			sipProvider.removeSipListener(this);
			sipStack.deleteSipProvider(sipProvider);
			sipStack.deleteListeningPoint(listeningPoint);
			sipStack.stop();
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}
	
	/**
	 * Arret du dialogue avec libration des ressources et remise 
	 * de l'agent dans un tat disponible
	 * @param transaction conduisant  l'arret
	 */	 
	protected void terminate(Transaction transaction) {
		try {
			Dialog dialog = transaction.getDialog();
			if (dialog != null) {
				dialog.delete();
			}
			transaction.terminate();
			currentDialog = null;
			currentServerTransaction = null;
			currentClientTransaction = null;
		} catch (Exception e) {
			e.printStackTrace();
		}
			
	} 

	
	
	/**
	 * Methode permettant de recuperer l'interface graphique ecoutant les evenements renvoyes par le UserAgent
	 * @param newListener
	 */
	public void setListener(UserAgentListener newListener) {
		listener = newListener;
	}

	/* (non-Javadoc)
	 * @see javax.sip.SipListener#processRequest(javax.sip.RequestEvent)
	 */
	public void processRequest(RequestEvent requestEvent) {
		
		Request req = requestEvent.getRequest();
		logReceivedRequest(req);
		
		//On rcupre la Transaction Serveur associe  la requte, si elle n'existe pas on la cre
		ServerTransaction serverTransaction = requestEvent.getServerTransaction();
		if (serverTransaction == null) {
			try {
				serverTransaction = sipProvider.getNewServerTransaction(req);
				serverTransaction.setRetransmitTimer(2000);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		
		//INVITE
		if (req.getMethod().equals(Request.INVITE)) {
			processInvite(req, serverTransaction);
		} 

		//ACK
        else if (req.getMethod().equals(Request.ACK)) {
        	processAck(req, serverTransaction);
        }

		//BYE
        else if (req.getMethod().equals(Request.BYE)) {
			processBye(req, serverTransaction);
        }

		//CANCEL
        else if (req.getMethod().equals(Request.CANCEL)) {
			processCancel(req, serverTransaction);
        }

		//OPTIONS
        else if (req.getMethod().equals(Request.OPTIONS)) {
			processOption(req, serverTransaction);
        }
        else {
        	// Cas requte non traite
        }
	}
	
	/**
	 * Traite une requte d'initiation d'appel
	 * @param newInvite Requte INVITE
	 * @param newTransaction Transaction lie  la requte
	 */
	protected void processInvite(Request newInvite, ServerTransaction newTransaction) {
		try {

			CallIdHeader callIDHeader = (CallIdHeader) newInvite.getHeader(CallIdHeader.NAME);
			CSeqHeader cSeqHeader = (CSeqHeader) newInvite.getHeader(CSeqHeader.NAME);
			FromHeader fromHeader = (FromHeader) newInvite.getHeader(FromHeader.NAME);
			
			//Rponse  un message INVITE 
			if (currentDialog == null) {
				currentServerTransaction = newTransaction;
				methods.put(cSeqHeader.getSeqNumber(), Request.INVITE);
				currentDialog = newTransaction.getDialog();

				// Envoi de message RINGING
				startRinging(newInvite, newTransaction);
				
				notifyListener(new NewCallEvent(newInvite, fromHeader.getAddress().toString(), newTransaction));
			} else {
				//Si on reoit un appel et que l'on a dj un appel en cours
				if (!currentDialog.getCallId().equals(callIDHeader.getCallId())) {
					Response res = messageFactory.createResponse(Response.BUSY_HERE, newInvite);
					res.addHeader(allowHeader);
					newTransaction.sendResponse(res);
					logSentResponse(res);
				}
				// sinon on ignore
			}
		}
		catch (Exception e) {
			e.printStackTrace();
		}
	}

	protected class RingingTask extends TimerTask {
		Response ringingResponse; 
		ServerTransaction newTransaction; 
		
		public RingingTask(Request req, ServerTransaction st) throws ParseException {
			newTransaction = st;
			
			ringingResponse = messageFactory.createResponse(Response.RINGING, req);
			ToHeader toHeader = (ToHeader) ringingResponse.getHeader(ToHeader.NAME);
			//gnration du LocalTag serveur (diffrent du client)
			toHeader.setTag(generateLocalTag());

			ContactHeader contactHeader = headerFactory.createContactHeader(myLogicalAddress);
			ringingResponse.addHeader(contactHeader);
			ringingResponse.addHeader(allowHeader);
		}

		@Override
		public void run() {
			System.out.println("Send Ringing Response");
			try {
				newTransaction.sendResponse(ringingResponse);
				logSentResponse(ringingResponse);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}	
	}

	protected void startRinging(Request newInvite,
			ServerTransaction newTransaction) throws ParseException,
			SipException, InvalidArgumentException {
		ringingTimer = new Timer();
		ringingTimer.schedule(new RingingTask(newInvite, newTransaction), RINGING_START, RINGING_DELAY);		
	}

	protected void stopRinging() {
		if (ringingTimer != null) {
			ringingTimer.cancel();
		}
		ringingTimer = null;
	}
	
	
	/**
	 * Traite une requte d'accus de rception
	 * @param newAck Requte ACK
	 * @param newTransaction Transaction lie  la requte
	 */
	protected void processAck(Request newAck, ServerTransaction newTransaction) {
    	CSeqHeader cSeqHeader = (CSeqHeader) newAck.getHeader(CSeqHeader.NAME);
    	//Recuperation de la requte  l'origine de l'ACK
    	String method = (String) methods.get(cSeqHeader.getSeqNumber());
    	    	
    	// Vrification que ce ACK correspond bien au dialog courant 
    	if (newTransaction.getDialog() == currentDialog) {
    		// Si c'est un ACK d'une transaction ayant dbut par un INVITE, on dmarre le transmiter 
    		// et on remonte un vnement de connexion  l'UI
    		 if (method.equals(Request.INVITE)) {
    			try {
    				transmiter.start();
    				notifyListener(new ConnectedEvent());
    			} catch (Exception e) {
    				e.printStackTrace();
    			}
    		}  
    	}
	}

	/**
	 * Traite une requete de fin d'appel
	 * @param newBye Requete Bye
	 * @param newTransaction Transaction liee a la requte
	 */
	protected void processBye(Request newBye, ServerTransaction newTransaction) {
    	
		if (newTransaction.getDialog() == currentDialog) {
			try {
				
				Response res = messageFactory.createResponse(Response.OK, newBye);
				res.addHeader(allowHeader);
				newTransaction.sendResponse(res);
				logSentResponse(res);
				
			} catch (Exception e) {
				e.printStackTrace();
			}

			// dconnexion rtp
			destroyRTP();
	    	// on remet l'agent dans un tat disponible
	    	terminate(newTransaction);   	
	    	//Remonte un vnement de dconnexion  l'UI
	    	notifyListener(new DisconnectedEvent());			
    	}
		// sinon on ignore
    	
	}

	/**
	 * Traite une requte d'annulation d'appel
	 * @param newCancel Requte Cancel
	 * @param newTransaction Transaction lie  la requte
	 */
	protected void processCancel(Request newCancel, ServerTransaction newTransaction) {
    	try {
    		// 200 OK
			Response res = messageFactory.createResponse(Response.OK, newCancel);
			res.addHeader(allowHeader);
			newTransaction.sendResponse(res);
			logSentResponse(res);
			stopRinging();
			
			if (currentServerTransaction!=null) {
				// 487 Request Terminated
				Response res2 = messageFactory.createResponse(Response.REQUEST_TERMINATED, currentServerTransaction.getRequest());
				currentServerTransaction.sendResponse(res2);
				logSentResponse(res2);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		//Libration de la "ligne"
    	terminate(newTransaction);
		//On remonte l'vnement d'annulation  l'UI
		notifyListener(new CancelledCallEvent());
		
	}


	/**
	 * Traite une requete OPTION
	 * @param newOption
	 * @param newTransaction
	 */
	protected void processOption(Request newOption, ServerTransaction newTransaction) {
		try {
			Response res = messageFactory.createResponse(Response.OK, newOption);
			res.addHeader(allowHeader);
			newTransaction.sendResponse(res);
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}
	
	/* (non-Javadoc)
	 * @see javax.sip.SipListener#processResponse(javax.sip.ResponseEvent)
	 */
	public void processResponse(ResponseEvent responseEvent) {
		Response res = responseEvent.getResponse();
		int statusCode = res.getStatusCode();
		
		logReceivedResponse(res);
		
		//Rcupration de la transaction courante
		ClientTransaction transaction = responseEvent.getClientTransaction();
		
		//OK
		if (statusCode == Response.OK) {
			processOK(res, transaction);
		}
		//Ringing
        else if (statusCode == Response.RINGING) {
        	processRinging(res, transaction);
        }
		//Trying
        else if (statusCode == Response.TRYING || statusCode == Response.REQUEST_TERMINATED) {
        	// ignore
        }
        else if (statusCode == Response.TEMPORARILY_UNAVAILABLE) {
        	processTemporarilyUnavailable(res, transaction);
        }
		//Trying
        else if (statusCode == Response.DECLINE) {
        	processDecline(res, transaction);
        }
        else if (statusCode == Response.DOES_NOT_EXIST_ANYWHERE) {
        	processDoesNotExistAnyWhere(res, transaction);
        }
        else if (statusCode == Response.BUSY_EVERYWHERE) {
        	processBusyEveryWhere(res, transaction);
        }
		//fail, server error, general fail
        else if (statusCode >= Response.SERVER_INTERNAL_ERROR) {
        	processError(res, transaction);
        }
	}


	protected void processTemporarilyUnavailable(Response res,
			ClientTransaction transaction) {
		terminate(transaction);
		notifyListener(new TemporarilyUnavailableEvent());	
	}


	private void processDecline(Response res, ClientTransaction transaction) {
		terminate(transaction);
		notifyListener(new DeclinedEvent());
	}


	protected void processDoesNotExistAnyWhere(Response res,
			ClientTransaction transaction) {	
		terminate(transaction);
		notifyListener(new DisconnectedEvent());
	}

	/**
	 * Traite une rponse correspond  une erreur serveur
	 * @param newOK
	 * @param newTransaction
	 */
	protected void processError(Response res, ClientTransaction transaction) {
		// On remet le useragent dans un tat disponible
	    terminate(transaction);
	    notifyListener(new DisconnectedEvent());
	}


	/**
	 * Traite une reponse OK
	 * @param newOK
	 * @param newTransaction
	 */
	protected void processOK(Response newOK, ClientTransaction newTransaction) {
		CSeqHeader  cSeqHeader = (CSeqHeader) newOK.getHeader(CSeqHeader.NAME);
		Dialog dialog = newTransaction.getDialog();
		Request request = newTransaction.getRequest();
		
		//Si le OK provient d'une requte INVITE
		if (request.getMethod().equals(Request.INVITE)) {
			try {
				Request ack = dialog.createAck(cSeqHeader.getSeqNumber());
				dialog.sendAck(ack);
				logSentRequest(ack);
				
				//On cre un RTPManager en fonction du flux audio support
				rtpManager = createManager(sdpFactory.createSessionDescription(new String(newOK.getRawContent())));
				//Avec ce manager on cre le receiver et le transmiter
				receiver = new AudioReceiver(rtpManager, logger);
				transmiter = new AudioTransmiter(rtpManager, logger);			

				//lancement du transmitter
				transmiter.start();
			}
			catch (Exception ex) {
				ex.printStackTrace();
			}
			notifyListener(new ConnectedEvent());
		}
		
		//Si le OK provient d'une requte BYE
		else if (request.getMethod().equals(Request.BYE)) {
			// Libration des ressources pour les flux audio
			destroyRTP();
			// on remet l'agent dans un tat disponible
			terminate(newTransaction);
			//On remonte un vnement de dconnexion  l'UI
			notifyListener(new DisconnectedEvent());			
		}
		
		//Si le OK provient d'une requete REGISTER
		else if (request.getMethod().equals(Request.REGISTER)) {
			
			ContactHeader contactHeader = (ContactHeader) request.getHeader(ContactHeader.NAME);
			//Si c'est un OK pour un dsenregistrement on arrete le Timer 
			// qui sert  se rengistrer automatiquement avant l'expiration
			final int expires = contactHeader.getExpires();	
			if (expires == 0) {
				if (registerTimer != null) {
					registerTimer.cancel();
					registerTimer = null;
				}
				notifyListener(new UnregisteredEvent());
			}
			//Sinon on active le Timer pour faire l'enregistrement
			else {		
				registerTimer = new Timer(); 
				TimerTask registerTask = new TimerTask() { 
					@Override
					public void run() {
						try {
							register(expires);
						} catch (Exception e) {
							e.printStackTrace();
						} 
					}
				};
				int delay = expires * 1000;
				registerTimer.schedule(registerTask, delay);
				notifyListener(new RegisteredEvent());
			};
		} 
		//Si le OK provient d'une requete CANCEL emis par cet agent
		else if (request.getMethod().equals(Request.CANCEL)) {
			// on remet l'agent dans un tat disponible
			terminate(newTransaction);
			//On remonte un vnement de dconnexion  l'UI
			notifyListener(new DisconnectedEvent());
		}
	}

	/**
	 * Traite une rponse Ringing
	 * @param newRinging
	 * @param newTransaction
	 */
	protected void processRinging(Response newRinging, ClientTransaction newTransaction) {
    	//On remonte l'vnement de Sonnerie  l'UI
		notifyListener(new RingingEvent());
	}

	 

	/**
	 * Traite une reponse Busy
	 * @param newBusy
	 * @param newTransaction
	 */
	protected void processBusyEveryWhere(Response newBusy, ClientTransaction newTransaction) {
		// on remet l'agent dans un tat disponible
		terminate(newTransaction);
		//On remonte un vnement de dconnexion  l'UI
		notifyListener(new BusyCallEvent());		
	}

	/* (non-Javadoc)
	 * @see javax.sip.SipListener#processTimeout(javax.sip.TimeoutEvent)
	 */
	public void processTimeout(TimeoutEvent timeoutEvent) {
		logger.trace("Timeout");
	}
	
	/* (non-Javadoc)
	 * @see javax.sip.SipListener#processDialogTerminated(javax.sip.DialogTerminatedEvent)
	 */
	public void processDialogTerminated(DialogTerminatedEvent dialogTerminatedEvent) {
		logger.trace("DialogTerminated");
	}

	/* (non-Javadoc)
	 * @see javax.sip.SipListener#processTransactionTerminated(javax.sip.TransactionTerminatedEvent)
	 */
	public void processTransactionTerminated(TransactionTerminatedEvent transactionTerminatedEvent) {
		logger.trace("TransactionTerminated");
	}
	
	/* (non-Javadoc)
	 * @see javax.sip.SipListener#processIOException(javax.sip.IOExceptionEvent)
	 */
	public void processIOException(IOExceptionEvent exceptionEvent) {
		System.out.println("IOException");
		
	}

	/**
	 * Gnre alatoirement un localTag  insrer dans les messages SIP
	 * @return
	 */
	public static synchronized String generateLocalTag() {
		return Integer.toHexString(localTagGenerator.nextInt());
	}
	
	/**
	 * Remonte les vnements  l'UI
	 * @param uae
	 */
	protected void notifyListener(UserAgentEvent uae) {
		if (listener != null) {
			listener.notify(uae);
		}
	}

	/**
	 * Cre un RTPManager en fonction du port et l'adresse rcuprs grce  la variable sdpContent
	 * @param sdpContent Contenu du message SDP
	 * @return
	 * @throws Exception
	 */
	protected RTPManager createManager(SessionDescription sdpContent) throws Exception {
		RTPManager manager = RTPManager.newInstance();

		InetAddress localInet = InetAddress.getByName(listeningPoint.getIPAddress());
		InetAddress destInet = InetAddress.getByName(sdpContent.getConnection().getAddress());
		
		int port = 0, i = 0;
		Vector medias = sdpContent.getMediaDescriptions(false);
		if (medias != null) {
			//On analyse les diffrents mdias supports jusqu' trouv un media de type audio
			while (i<medias.size() && port == 0) {
				MediaDescription mediaDesc = (MediaDescription) medias.elementAt(i);
				Media media = mediaDesc.getMedia();
				if (media.getMediaType().equals("audio")) {
					port = media.getMediaPort();
					System.out.println("port audio trouve: " + port);
				}
				i++;
			}
		}
		
		if (port == 0)
			throw new Exception("AudioPort not found");
		
		//Initialisation de l'adresse locale et ditante pour l'mission et la rception des flux
		SessionAddress localAddress = new SessionAddress(localInet, rtpPort);
		SessionAddress toAddress = new SessionAddress(destInet, port);
		manager.initialize(localAddress);
		manager.addTarget(toAddress);
		
		return manager;
	}

	/**
	 * Libration des ressources li aux flux audio
	 */
	public void destroyRTP() {
		if (transmiter != null)
			transmiter.stop();
		if (receiver != null)
			receiver.close();
		if (rtpManager != null) {
			rtpManager.removeTargets("Closing");
			rtpManager.dispose();
		}
		transmiter = null;
		receiver = null;
		rtpManager = null;
	}

	
	/**
	 * Cration d'un contenu SDP pour les messages INVITE SIP
	 * @return
	 * @throws SdpException
	 */
	public SessionDescription createSDPContent() throws SdpException {
		
		SessionDescription sdpContent = sdpFactory.createSessionDescription();
		
		// attribut Origin: o = 
		Origin origin = sdpFactory.createOrigin("-", listeningPoint.getIPAddress());
		sdpContent.setOrigin(origin);
		
		//attribut Connection : c = IN IP4 IP address
		Connection connection = sdpFactory.createConnection(listeningPoint.getIPAddress());
		sdpContent.setConnection(connection);
		
		//attribut Session : s=
		SessionName sessionName = sdpFactory.createSessionName("TL1 SIP Session");
		sdpContent.setSessionName(sessionName);
		
		// Contenu SDP
		Vector<Field> sdpFields = new Vector<Field>();
		Vector<Integer> rtpAvpTypes = new Vector<Integer>();
		rtpAvpTypes.add(0);
		sdpFields.add(sdpFactory.createMedia("audio", rtpPort, 0, "RTP/AVP", rtpAvpTypes));
		sdpFields.add(sdpFactory.createAttribute("rtpmap", "0 PCMU/8000"));
		
		sdpContent.setAttributes(sdpFields);
		return sdpContent;
	}

	
	/**
	 * Envoi d'une requte INVITE (mthode appele depuis l'UI
	 * @param  URI de destination
	 * @throws ParseException
	 * @throws InvalidArgumentException
	 * @throws SipException
	 */
	public void invite(String destination) throws ParseException, InvalidArgumentException, SipException {
		
		URI toURI = addressFactory.createURI(destination);
		
		// Creation d'entete Message
		CallIdHeader callIDHeader = sipProvider.getNewCallId();
		CSeqHeader cseqHeader = headerFactory.createCSeqHeader((++cSeq), Request.INVITE);
		MaxForwardsHeader maxForwards = headerFactory.createMaxForwardsHeader(70);

		// Creation d'entete From et Contact
		FromHeader fromHeader = headerFactory.createFromHeader(myLogicalAddress, generateLocalTag());
		ContactHeader contactHeader = headerFactory.createContactHeader(myPhysicalAddress);

		// Creation d'entete To
		Address toAddress = addressFactory.createAddress(toURI);
		ToHeader toHeader = headerFactory.createToHeader(toAddress, null);
		
		// Creation d'entete Via		
		String transport = "udp";
		ArrayList<ViaHeader> viaHeaders = new ArrayList<ViaHeader>();
		ViaHeader viaHeader = headerFactory.createViaHeader(listeningPoint.getIPAddress(), listeningPoint.getPort(), transport, null);
		viaHeaders.add(viaHeader);

		
		// Creation Message
		Request invite = messageFactory.createRequest(toAddress.getURI(), Request.INVITE, callIDHeader, cseqHeader, fromHeader, toHeader, viaHeaders, maxForwards);
		invite.addHeader(contactHeader);
		invite.addHeader(allowHeader);
		
		
		//SDP
		ContentTypeHeader contentTypeHeader = headerFactory.createContentTypeHeader("application", "sdp");
		try {
			invite.setContent(createSDPContent(), contentTypeHeader);
		} catch (SdpException e) {
			e.printStackTrace();
		}
		
		// cration de la transaction
		ClientTransaction clientTransaction = sipProvider.getNewClientTransaction(invite);
		currentClientTransaction = clientTransaction;
		currentDialog = clientTransaction.getDialog();
		// Envoi Message
		clientTransaction.sendRequest();
		logSentRequest(invite);
	}
	
	/**
	 * Annuler un appel (mthode appele depuis l'UI)
	 */
	public void cancel() {
		try {
			Request cancelRequest = currentClientTransaction.createCancel();
			cancelRequest.addHeader(allowHeader);
		    ClientTransaction clientTransaction = sipProvider.getNewClientTransaction(cancelRequest);
		    clientTransaction.sendRequest();
		    logSentRequest(cancelRequest);
		    
		} catch (SipException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Accepter un appel (mthode appele depuis l'UI)
	 * @param newCallEvent variable permettant de passer des paramtres venant de l'UI
	 * @throws ParseException
	 * @throws SdpException
	 * @throws SipException
	 * @throws InvalidArgumentException
	 */
	public void accept(NewCallEvent newCallEvent) throws ParseException, SdpException, SipException, InvalidArgumentException{
		
		stopRinging();
		
		//Cration du rtpManager
		try {
			rtpManager = createManager(sdpFactory.createSessionDescription(new String(newCallEvent.getInvite().getRawContent())));
		} catch (Exception e) {
			e.printStackTrace();
		}
		transmiter = new AudioTransmiter(rtpManager ,logger);
		receiver = new AudioReceiver(rtpManager, logger);

		Response res = messageFactory.createResponse(Response.OK, newCallEvent.getInvite());

		ContactHeader contactHeader = headerFactory.createContactHeader(myLogicalAddress);
		res.addHeader(contactHeader);
		res.addHeader(allowHeader);
		
		//ajout de la description SDP au corps du message SIP
		ContentTypeHeader contentTypeHeader = headerFactory.createContentTypeHeader("application", "sdp");	
		res.setContent(createSDPContent(), contentTypeHeader);
		
		newCallEvent.getTransaction().sendResponse(res);
		logSentResponse(res);
	}

	/**
	 * Refuser un appel (mthode appele depuis l'UI)
	 * @param newCallEvent variable permettant de passer des paramtres venant de l'UI
	 * @throws ParseException
	 * @throws SdpException
	 * @throws SipException
	 * @throws InvalidArgumentException
	 */
	public void decline(NewCallEvent newCallEvent) throws ParseException, SdpException, SipException, InvalidArgumentException{
		
		stopRinging();
		
		Response res = messageFactory.createResponse(Response.DECLINE, newCallEvent.getInvite());

		ContactHeader contactHeader = headerFactory.createContactHeader(myLogicalAddress);
		res.addHeader(contactHeader);
		res.addHeader(allowHeader);
		
		newCallEvent.getTransaction().sendResponse(res);
		logSentResponse(res);
		
		// on remet l'agent dans un tat disponible
		terminate(newCallEvent.getTransaction());
	}

	/**
	 * Terminer un appel (mthode appele depuis l'UI)
	 * @throws ParseException
	 * @throws InvalidArgumentException
	 * @throws SipException
	 */
	public void bye() throws ParseException, InvalidArgumentException, SipException{
		ContactHeader contactHeader = headerFactory.createContactHeader(myPhysicalAddress);

		// Creation Message
		Request bye = currentDialog.createRequest(Request.BYE);
		bye.addHeader(contactHeader);
		bye.addHeader(allowHeader);

		// cration de la transaction
		ClientTransaction clientTransaction = sipProvider.getNewClientTransaction(bye);

		// Envoi Message
		currentDialog.sendRequest(clientTransaction);
		logSentRequest(bye);
	}
	
	/**
	 * S'enregistrer aupres d'un proxyAddress (mthode appele depuis l'UI)
	 * @param expires Temps au bout duquel l'enregistrement expire
	 * @throws InvalidArgumentException
	 * @throws ParseException
	 * @throws SipException
	 */
	public void register(int expires) throws InvalidArgumentException, ParseException, SipException{
		// Creation d'entete Message
		CallIdHeader callIDHeader = sipProvider.getNewCallId();
		CSeqHeader cseqHeader = headerFactory.createCSeqHeader((++cSeq), Request.REGISTER);
		MaxForwardsHeader maxForwards = headerFactory.createMaxForwardsHeader(70);
		
		// Creation d'entete From et Contact
		FromHeader fromHeader = headerFactory.createFromHeader(myLogicalAddress, generateLocalTag());
		ContactHeader contactHeader = headerFactory.createContactHeader(myPhysicalAddress);
		contactHeader.setExpires(expires);
		int arobasIndex = mail.indexOf('@');
		String userMail = mail.substring(0, arobasIndex);
		String domainMail = mail.substring(arobasIndex+1); 
		//contactHeader.setParameter("usermail", userMail);
		//contactHeader.setParameter("domainmail", domainMail);
		
		// Creation d'entete To
		ToHeader toHeader = headerFactory.createToHeader(myLogicalAddress, null);
		
		// Creation d'entete Via		
		String transport = "udp";
		ArrayList<ViaHeader> viaHeaders = new ArrayList<ViaHeader>();
		ViaHeader viaHeader = headerFactory.createViaHeader(listeningPoint.getIPAddress(), listeningPoint.getPort(), transport, null);
		viaHeaders.add(viaHeader);

		
		// Creation Message
		Request register = messageFactory.createRequest(proxyAddress.getURI(), Request.REGISTER, callIDHeader, cseqHeader, fromHeader, toHeader, viaHeaders, maxForwards);
		register.addHeader(contactHeader);
		register.addHeader(allowHeader);
		
		
		// cration de la transaction
		ClientTransaction clientTransaction = sipProvider.getNewClientTransaction(register);
			
		// Envoi Message
		clientTransaction.sendRequest();
		logSentRequest(register);
	}
	
	protected void logReceivedRequest(Request req) {
		logger.trace("Received request:\n" + req);
	}
	protected void logSentRequest(Request req) {
		logger.trace("Sent request:\n" + req);
	}
	
	protected void logReceivedResponse(Response resp) {
		logger.trace("Received response:\n" + resp);
	}
	protected void logSentResponse(Response resp) {
		logger.trace("Sent response:\n" + resp);
	}
	

}
