package livresip.sipphone.ui;

/** 
 * Cette classe gre les dialogues et transactions SIP.<br/>
 * Elle permet de crer des requetes/rponses en fonction des requetes/rponses reues.
 * @copyright Telecom Lille 1 
 */

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.net.InetAddress;
import java.util.Arrays;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.List;
import java.util.Map.Entry;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.LinkedList;
import java.util.Properties;
import java.util.Set;

import javax.sip.SipListener;
import javax.swing.*;
import javax.swing.border.TitledBorder;
import javax.swing.event.*;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;

import java.awt.*;
import java.awt.Dialog.ModalityType;
import java.awt.event.*;

import livresip.sipphone.events.BusyCallEvent;
import livresip.sipphone.events.CancelledCallEvent;
import livresip.sipphone.events.ConnectedEvent;
import livresip.sipphone.events.DeclinedEvent;
import livresip.sipphone.events.DisconnectedEvent;
import livresip.sipphone.events.MovedEvent;
import livresip.sipphone.events.NewCallEvent;
import livresip.sipphone.events.RegisteredEvent;
import livresip.sipphone.events.RingingEvent;
import livresip.sipphone.events.TemporarilyUnavailableEvent;
import livresip.sipphone.events.UnregisteredEvent;
import livresip.sipphone.events.UserAgentEvent;
import livresip.sipphone.media.Tone;
import livresip.sipphone.media.TonePlayer;
import livresip.sipphone.sip.UserAgentSIP;
import livresip.sipphone.sip.UserAgentListener;
import gov.nist.javax.sip.header.UserAgent;



public class UserAgentUI extends JFrame implements UserAgentListener {
	
	// attributs pour la gestion la configuration
	protected Properties config = null;
	protected String configFilename; 
	protected String localIP;
	
	// attributs pour la gestion de la communication
	protected enum UserAgentState {DISCONNECTED, CONNECTING, CONNECTED};
	protected NewCallEvent currentCall;
	protected UserAgentState currentState;
	protected boolean isRegistered;
	protected UserAgentSIP uaSip;
	
	// attributs pour l'interface utilisateur
	protected JDialog incomingDialog; 
	protected boolean hasCallSelection;
	protected boolean abortedDialog;
	
	protected JPanel sipTab;
	protected JPanel configTab;
	protected JPanel contactsTab; 
	
	protected JLabel lblRemoteIP;
	protected JTextField txtRemoteIP;
	protected JButton btConnect;
	protected JLabel lbCaller;
	protected JButton btAccept;
	protected JButton btDecline;
	protected JLabel lbState;
	protected JButton btRegister;
	protected JTabbedPane tabFolder;
	protected JTextArea infoTextArea;
	protected JScrollPane infoScrollPane;
	
	protected JLabel lblLogin;
	protected JTextField txtUser;
	protected JLabel lblDomain;
	protected JTextField txtDomain;
	protected JLabel lblMail;
	protected JTextField txtMail;
	protected Pattern mailPattern = Pattern.compile("^[A-Za-z0-9._%+-]+@[A-Za-z0-9.-]+\\.[A-Za-z]{2,4}$");

	protected JLabel lblRegister;
	protected JTextField txtProxy;
	protected JButton btSave;
	protected JLabel lblPort;
	protected JSpinner spPort;
	protected JLabel lblDuree;
	protected JSpinner spDuree;
	
	protected static final String USER_PROPERTY = "sip.phone.user";
	protected static final String DOMAIN_PROPERTY = "sip.phone.domain";
	protected static final String MAIL_PROPERTY = "sip.phone.mail"; 
	protected static final String REGISTRAR_PROPERTY = "sip.phone.registrar";
	protected static final String DURATION_PROPERTY = "sip.phone.duration";
	protected static final String PORT_PROPERTY = "sip.phone.port";

	protected static final String CONNECT_TXT = "Connect";
	protected static final String REGISTER_TXT = "Register";
	protected static final String UNREGISTER_TXT = "Unregister";
	protected static final String DISCONNECT_TXT = "Disconnect";
	protected static final String CANCEL_TXT = "Cancel Call";
	
	protected static final String CANCELED_TXT = "Canceled Call";
	protected static final String DISCONNECTED_TXT = "Disconnected";
	protected static final String CONNECTED_TXT = "Connected";
	protected static final String DOESNOTANSWER_TXT = "Callee did not answer";
	protected static final String MOVED_TXT = "Callee has moved to";
	protected static final String DECLINED_TXT = "Declined Call";
	protected static final String BUSY_TXT = "Busy Callee";
	
	protected boolean closing = false;

	protected Logger logger;
	protected JLabel lblContact;
	protected JList lstContacts;
	protected JComboBox cbxContact;
	protected JScrollPane scrollPane;
	protected JLabel lblContactName;
	protected JTextField txtContactName;
	protected JLabel lblContactAddress;
	protected JTextField txtContactAddress;
	protected JButton btAddContact;
	protected JButton btRemoveContact;

	
	protected TonePlayer tonePlayer;
	
	protected DefaultListModel contactsListModel;
	protected DefaultComboBoxModel contactsComboxModel;

	public UserAgentUI(String filename) {	
		this.configFilename = filename;
		
		this.initStateAndModel();
		this.buildUI();	
		this.loadConfig();
	    this.initLogger();
	    this.initAgent();
	    this.initTonePlayer();
	    
	    // open
	    this.pack();
	    this.setVisible(true);
	}
	
	/**
	 * Initialisation de l'tat et des modles pour les contacts
	 */
	protected void initStateAndModel() {
		isRegistered = false;
		currentState = UserAgentState.DISCONNECTED;

		contactsListModel = new DefaultListModel();
		contactsComboxModel = new DefaultComboBoxModel();
	
	}
	
	/**
	 * Initialisation du gestion de tonalits
	 */
	protected void initTonePlayer() {
		new Thread(new Runnable() { 
			public void run() {			
				try {
					tonePlayer = new TonePlayer();
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		}).start();
	}


	/**
	 * Charge la configuration depuis le fichier config.properties. 
	 * Fixe les paramtres par dfaut en cas d'erreur.
	 */
	protected void loadConfig() {
		FileInputStream fis = null; 
		config = new Properties();
		
		try {
			fis= new FileInputStream(configFilename);
			config.load(fis);
		} catch (Exception ex) {
			config.setProperty(USER_PROPERTY, System.getProperty("user.name"));
			config.setProperty(DOMAIN_PROPERTY, "mydomain.com");
			config.setProperty(MAIL_PROPERTY, "myemail.com");
			config.setProperty(REGISTRAR_PROPERTY, "proxyAddress");
			config.setProperty(DURATION_PROPERTY, "3600");
			config.setProperty(PORT_PROPERTY, "5070");
		}
		txtUser.setText(config.getProperty(USER_PROPERTY));
		txtDomain.setText(config.getProperty(DOMAIN_PROPERTY));
		txtMail.setText(config.getProperty(MAIL_PROPERTY));
		txtProxy.setText(config.getProperty(REGISTRAR_PROPERTY));
		spDuree.setValue(Integer.parseInt(config.getProperty(DURATION_PROPERTY)));
		spPort.setValue(Integer.parseInt(config.getProperty(PORT_PROPERTY)));  
		
		fillContactsFrom(config);
	}
	
	/** 
	 * Remplit la base de contact  partir des proprits lues du fichier
	 */
	protected void fillContactsFrom(Properties properties) {
		for (Entry<Object, Object> e:properties.entrySet()) {
			String contactKey = (String) e.getKey();
			if (contactKey.startsWith("contact")) {
				String key = contactKey.replaceFirst("contact.","");
				String name = key.replace("$", " ");
				String address = (String) e.getValue();
				Contact nc = new Contact(name, address);
				contactsListModel.addElement(nc);
				contactsComboxModel.addElement(nc);
			}
		}
	}
	
	/** 
	 * Remplit les proprits  partir de la base de contact
	 */
	protected void fillContactsTo(Properties properties) {
		Enumeration en = contactsListModel.elements();
		while(en.hasMoreElements()) {
			Contact c = (Contact) en.nextElement();
			String name = c.getName().replace(" ", "$");
			String address = c.getAddress();
			String key = "contact." + name;
			properties.put(key, address);
		}
	}
	
	/**
	 * Sauve la configuration dans le fichier config.properties
	 */
	protected void saveConfig() {
		config.setProperty(USER_PROPERTY, txtUser.getText());
		config.setProperty(DOMAIN_PROPERTY, txtDomain.getText());
		config.setProperty(MAIL_PROPERTY, txtMail.getText());
		config.setProperty(REGISTRAR_PROPERTY, txtProxy.getText());
		config.setProperty(DURATION_PROPERTY, spDuree.getValue().toString());
		config.setProperty(PORT_PROPERTY, spPort.getValue().toString());
		
		fillContactsTo(config);
		try {
			String filename = (configFilename != null) ? configFilename : "config.properties";
			FileOutputStream fos = new FileOutputStream(filename);
			config.store(fos, "Automatically built configuration");
		} catch (Exception e) {
			e.printStackTrace();
		} 
	}
	
	
	/**
	 * Initialisation de l'agent
	 */
	protected void initAgent() {
		try {
			if (uaSip != null) {
				uaSip.stop();
			}
			localIP = InetAddress.getLocalHost().getHostAddress();
			uaSip = new UserAgentSIP(
					localIP,
					Integer.parseInt(config.getProperty(PORT_PROPERTY)),
					"udp",
					config.getProperty(USER_PROPERTY),
					config.getProperty(DOMAIN_PROPERTY), 
					config.getProperty(MAIL_PROPERTY),
					config.getProperty(REGISTRAR_PROPERTY),
					logger); 		
			uaSip.setListener(this);
		}
		catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * Initialisation du logger
	 */
	 public void initLogger() {
		logger = Logger.getLogger(this.getClass().getName());
		logger.setLevel(Level.TRACE); 
		 
		// configuration du log 
		TextAreaAppender.setTextArea(infoTextArea);
			
		Properties logProperties = new Properties();
		logProperties.put("log4j.rootLogger", "TRACE, TEXTAREA");
		logProperties.put("log4j.appender.TEXTAREA", "livresip.sipphone.ui.TextAreaAppender");  // Our custom appender
		logProperties.put("log4j.appender.TEXTAREA.layout", "org.apache.log4j.PatternLayout"); //See: http://logging.apache.org/log4j/docs/api/org/apache/log4j/PatternLayout.html
		logProperties.put("log4j.appender.TEXTAREA.layout.ConversionPattern", "%d{HH:mm:ss} %5.5p %40.40c: %m%n");
			
		PropertyConfigurator.configure(logProperties);
		 
	 }
	
	
	/**
	 * Cration de conteneur d'onglet
	 */
	protected void createTabFolder() {
		tabFolder = new JTabbedPane();
		createSipTab();
		createConfigTab();
		createContactsTab();
		
		tabFolder.add("SIP", sipTab);
		tabFolder.add("Config", configTab);
		tabFolder.add("Contacts", contactsTab);

	}

	
	/**
	 * Cration de l'onglet Sip
	 */
	protected void createSipTab() {
		
		sipTab = new JPanel();
		sipTab.setLayout(new BoxLayout(sipTab, BoxLayout.Y_AXIS));
		
		lblRemoteIP = new JLabel("Callee Sip address:");
		txtRemoteIP = new JTextField(20);
		lblContact = new JLabel("Contacts List");
		cbxContact = new JComboBox(contactsComboxModel);
		
		cbxContact.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				Contact c = (Contact) cbxContact.getSelectedItem();
				if (c != null) {
					txtRemoteIP.setText("sip:" + c.getAddress());
				}
			}
			
		});
		
		btConnect = new JButton(CONNECT_TXT);
		lbState = new JLabel(DISCONNECTED_TXT);
		
		// Gestion du click sur le bouton "Connect"

		btConnect.addActionListener(new ActionListener()  {
			public void actionPerformed(ActionEvent e) {
				// appel de la mthode invite
				if(currentState == UserAgentState.DISCONNECTED){
					try {
						currentState = UserAgentState.CONNECTING;
						uaSip.invite(txtRemoteIP.getText());
						btConnect.setText(CANCEL_TXT);
					}
					catch (Exception ex) {
						ex.printStackTrace();
					}
				}
				// si on est en attente de dcroch, on annule l'invitation
				else if (currentState == UserAgentState.CONNECTING) {
					try {
						uaSip.cancel();
					} catch (Exception ex) {
						ex.printStackTrace();
					}
					btConnect.setText(CONNECT_TXT);
					lbState.setText(DISCONNECTED_TXT);
				}
				// si on est en communication, on ferme l'appel
				else if (currentState == UserAgentState.CONNECTED) {
					// if(btConnect.getText().equals(DISCONNECT_TXT)){
					try {
						uaSip.bye();
					} catch (Exception ex) {
						ex.printStackTrace();
					}
				}
			}
		});
		
		btRegister = new JButton(REGISTER_TXT);
		// gestion du clic sur le bouton "Register"
		btRegister.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				try {
					// si on n'est pas enregistr, on s'enregistre
					if (! isRegistered) {
						uaSip.register(Integer.parseInt(config.getProperty(DURATION_PROPERTY)));
					}
					// on se dsenregistre sinon
					else {
						uaSip.register(0);
					}
				} catch (Exception ex) {
					ex.printStackTrace();
				}
			}
		});
		
		infoTextArea = new JTextArea(30, 128);
		infoTextArea.setFont(new Font("Monospaced", Font.PLAIN, 12));
		infoTextArea.setText("");
		infoTextArea.setBorder(BorderFactory.createLineBorder(Color.BLUE));
		
		infoScrollPane = new JScrollPane(infoTextArea);
		
		sipTab.add(lblRemoteIP);
		sipTab.add(txtRemoteIP);
		sipTab.add(lblContact);
		sipTab.add(cbxContact);
		sipTab.add(lbState);		
		sipTab.add(btConnect);
		sipTab.add(btRegister);
		
		sipTab.add(infoScrollPane);
		
	}


	/**
	 * Cration de l'onglet de configuration 
	 */
	protected void createConfigTab() {
		
		configTab = new JPanel();
		configTab.setLayout(new BoxLayout(configTab, BoxLayout.Y_AXIS));
		
		lblLogin = new JLabel("user name :");
		txtUser = new JTextField(20);
		lblDomain = new JLabel("domain :");
		txtDomain = new JTextField(20);
		lblMail = new JLabel("mail:");
		txtMail = new JTextField(30);
		lblRegister = new JLabel("Registrar server:");
		txtProxy = new JTextField(20);
		lblDuree = new JLabel("Duration");
		
		spDuree = new JSpinner(new SpinnerNumberModel(3600, 0, 86400, 1));		
		lblPort = new JLabel("Port :");
	
		spPort = new JSpinner(new SpinnerNumberModel(5060, 0, 65535, 1));
		btSave = new JButton("Save");
		
		
		// gestion du clic sur le bouton "Save"
		btSave.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				// il faut se dsenregistrer avant de sauver.
				if (btRegister.getText().equals(UNREGISTER_TXT)) {
					try {
						uaSip.register(0);
					} catch (Exception ex) {
						ex.printStackTrace();
					}					
					btRegister.setText(REGISTER_TXT);
				}
				// sauve la configuration
				saveConfig();
				// initialise l'agent 
				initAgent();
				// retour  l'onglet principal
				tabFolder.setSelectedIndex(0);				
			}
		});
		
		txtMail.addActionListener(new ActionListener() {
		    public void actionPerformed(ActionEvent e) {
		    	validateMail();       
		     }
		});
		
		txtMail.addFocusListener(new FocusListener() {
			public void focusLost(FocusEvent arg0) {
				validateMail();	
			}
			public void focusGained(FocusEvent arg0) {	}
		});
		
		configTab.add(lblLogin);
		configTab.add(txtUser);
		configTab.add(lblDomain);
		configTab.add(txtDomain);
		configTab.add(lblMail);
		configTab.add(txtMail);
		configTab.add(lblRegister);
		configTab.add(txtProxy);
		configTab.add(lblDuree);
		configTab.add(spDuree);
		configTab.add(lblPort);
		configTab.add(spPort);
		configTab.add(btSave);
	}
	
	/*
	 * Validation du mail fourni par l'utilisateur
	 */
	protected void validateMail() {
		Matcher matcher = mailPattern.matcher(txtMail.getText());
    	if (!matcher.matches()) {
                JOptionPane.showMessageDialog(null,
                			"Error: Please enter a correct email", 
                			"Error Message",
                			JOptionPane.ERROR_MESSAGE);
         }
	}
	
	
	/**
	 * Cration de l'onglet de configuration 
	 */
	protected void createContactsTab() {
		
		contactsTab = new JPanel();
		contactsTab.setLayout(new BoxLayout(contactsTab, BoxLayout.Y_AXIS));
		
		lstContacts = new JList(contactsListModel);
		scrollPane = new JScrollPane(lstContacts);
		scrollPane.setBorder(new TitledBorder("Contacts List"));
		
		lblContactName = new JLabel("Contact name :");
		txtContactName = new JTextField(20);
		lblContactAddress = new JLabel("Contact address :");
		txtContactAddress = new JTextField(20);
			
		btAddContact = new JButton("Add");
		btRemoveContact = new JButton("Remove");
		
		// gestion du clic sur le bouton "Add"
		btAddContact.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				String name = txtContactName.getText();
				String address = txtContactAddress.getText();
				
				if (!name.equals("") && !address.equals("")) {
					// check that URI is SIP compliant
					if (uaSip.isValidSipAddress(address)) {
						Contact contact = getContact(name);
						if (contact != null) {
							contactsListModel.removeElement(contact);
							contactsComboxModel.removeElement(contact);
						}
						contact = new Contact(name, address);
						contactsListModel.addElement(contact);
						contactsComboxModel.addElement(contact);
					}
				}
				// sauve la configuration
				saveConfig();
			}
		});
		
		btRemoveContact.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				Object[] selectedValues = lstContacts.getSelectedValues();
				for (Object c:selectedValues) {
					contactsListModel.removeElement(c);
					contactsComboxModel.removeElement(c);
				}	
			}
			
		});
		
		lstContacts.addListSelectionListener(new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent e) {
				Contact c = (Contact) lstContacts.getSelectedValue();
				if (c != null) {
					txtContactName.setText(c.getName());
					txtContactAddress.setText(c.getAddress());
				} 
				else {
					txtContactName.setText("");
					txtContactAddress.setText("");
				}
			}
		});
		
		contactsTab.add(scrollPane);
		contactsTab.add(lblContactName);
		contactsTab.add(txtContactName);
		contactsTab.add(lblContactAddress);
		contactsTab.add(txtContactAddress);
		contactsTab.add(btAddContact);
		contactsTab.add(btRemoveContact);
	}
	
	/**
	 * Construction de la fentre principale
	 */
	protected void buildUI() {
		this.setTitle("User Agent");
		this.setLayout(new GridLayout(1,1));
		this.setSize(200, 260);
		
		createTabFolder();
		this.setContentPane(tabFolder);
		
		// gestion de la cloture de la fenetre
		this.addWindowListener(new WindowAdapter() { 
			public void windowClosed(WindowEvent e) {
				closing = true;

				// Raccrocher si l'utilisateur quitte l'application pendant une conversation
				if(currentState == UserAgentState.CONNECTED) {
					try {
						uaSip.bye();
					} catch (Exception ex) {
						ex.printStackTrace();
					}
				}
				// Annuler l'invitation
				else if (currentState == UserAgentState.CONNECTING) {
					try {
						uaSip.cancel();
					} catch (Exception ex) {
						ex.printStackTrace();
					}
				}
				
				// se dsenregistrer
				if (isRegistered) {
					try {
						uaSip.register(0);
					} catch (Exception ex) {
						ex.printStackTrace();
					}
				}
				
				// libere les ressources des sonneries
				tonePlayer.close();

				// laisse la pile le temps d'envoyer les requetes
				try {
					Thread.sleep(300);
					uaSip.stop();
				} catch (Exception ex) {
					ex.printStackTrace();
				}

				System.exit(0);
			}
		});
	}
	
	protected Contact getContact(String name) {
		Contact result = null;
		Enumeration<?> elements = contactsListModel.elements();
		while(elements.hasMoreElements()) {
			Contact c = (Contact) elements.nextElement();
			if (name.equals(c.getName())) {
				result = c;
				break;
			}
		}
		return result;
	}
	
	protected Object getElementAt(int index) {
		return contactsListModel.get(index);
	}


	protected int getContactsSize() {
		return contactsListModel.size();
	}

	
	public static void main (String [] args) {
		final String configFilename;
		if (args.length != 1) {
			System.err.println("ce programme a besoin d'un nom de fichier de configuration");
			configFilename = "";
		} else {
			configFilename = args[0];
		}
		SwingUtilities.invokeLater(new Runnable() {
				public void run() {
					UserAgentUI agentUI = new UserAgentUI(configFilename);
				}
		});
	}
	
	/**
	 * Cration de la fenetre d'un nouvel appel entrant 
	 */
	protected synchronized void incomingCallDialog() {
		
		GridLayout gridLayout = new GridLayout(2, 1);

		incomingDialog = new JDialog(this, "Incoming Call", true); 
		incomingDialog.setLayout(gridLayout);
		incomingDialog.setSize(350, 85);
	
		hasCallSelection = false;
		abortedDialog = false;
		
		// gestion de la cloture de la fentre, on refuse l'appel
		incomingDialog.addWindowListener(new WindowAdapter() { 
			public void windowClosed(WindowEvent e) {
					try {
						if (! hasCallSelection && !abortedDialog) {
							uaSip.decline(currentCall);
						}
						tonePlayer.stopTones();
					} catch (Exception ex) {
						ex.printStackTrace();
					}
					
				}
		}); 
		
		lbCaller = new JLabel(currentCall.getCaller());
		incomingDialog.add(lbCaller);
		
		JPanel btnPanel = new JPanel();
		incomingDialog.add(btnPanel);
		
		btAccept = new JButton("Accept");
		btnPanel.add(btAccept);
			
		btDecline = new JButton("Decline");
		btnPanel.add(btDecline);
		
		// gestion du bouton "Accept"
		btAccept.addMouseListener(new MouseInputAdapter() {
			public void mouseClicked(MouseEvent e) {
				try {
					uaSip.accept(currentCall);
					tonePlayer.stopTones();
					hasCallSelection = true;
					incomingDialog.dispose();
					incomingDialog = null;
				} catch (Exception ex) {
					ex.printStackTrace();
				}
			}
		});
		
		// gestion du bouton "Decline"
		btDecline.addMouseListener(new MouseInputAdapter() {
			public void mouseClicked(MouseEvent e) {
				try {
					uaSip.decline(currentCall);
					tonePlayer.stopTones();
					hasCallSelection = true;
					incomingDialog.dispose();
					incomingDialog = null;
				} catch (Exception ex) {
					ex.printStackTrace();
				}
			}
		});
		
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				incomingDialog.setVisible(true);
			}
		});
	}
	
	
	/**
	 * Traitement d'un vnement de la couche SIP
	 */
	public synchronized void notify(UserAgentEvent uae) {
		 
		 if (!closing) {
			//Appel entrant
			if (uae instanceof NewCallEvent) {
				currentCall = (NewCallEvent) uae;
				incomingCallDialog();
				tonePlayer.playTone(TonePlayer.NEWCALL);
			}
			//Tonalit sonnerie
			else if (uae instanceof RingingEvent){
				lbState.setText("Calling ...");
				btConnect.setText(CANCEL_TXT);
				tonePlayer.playTone(TonePlayer.RINGING);
			}
			//Dconnect
			else if (uae instanceof DisconnectedEvent){
					disconnected(DISCONNECT_TXT);
					if (incomingDialog!=null)
						incomingDialog.dispose();
					tonePlayer.playTone(TonePlayer.BUSY);
			}
			//Appel annul
			else if (uae instanceof CancelledCallEvent){
					disconnected(CANCELED_TXT);
					if (incomingDialog!=null) {
						abortedDialog = true;
						incomingDialog.dispose();
					}
					tonePlayer.stopTones();
			}
			//Connect
			else if (uae instanceof ConnectedEvent){
					currentState = UserAgentState.CONNECTED;
					lbState.setText(CONNECTED_TXT);
					btConnect.setText(DISCONNECT_TXT);
					tonePlayer.stopTones();
			}
			// Occup
			else if (uae instanceof BusyCallEvent){
					disconnected(BUSY_TXT);
					tonePlayer.playTone(TonePlayer.BUSY);
			}
			// Refus
			else if (uae instanceof DeclinedEvent){
					disconnected(DECLINED_TXT);
					tonePlayer.playTone(TonePlayer.BUSY);
			}
			// Pas de dcrochage
			else if (uae instanceof TemporarilyUnavailableEvent){
					disconnected(DOESNOTANSWER_TXT); 
					tonePlayer.playTone(TonePlayer.BUSY);
			}
			// Deplac
			else if (uae instanceof MovedEvent){
					// TODO A completer avec la nouvelle addresse
					disconnected(MOVED_TXT);	
					tonePlayer.playTone(TonePlayer.BUSY);
			}
			//Registered
			else if (uae instanceof RegisteredEvent){
					isRegistered = true;
					btRegister.setText(UNREGISTER_TXT);
			}
			//Unregistered
			else if (uae instanceof UnregisteredEvent){
					isRegistered = false;
					btRegister.setText(REGISTER_TXT);
			}
		}
	}
	
	protected void disconnected(String reason) {
		currentState = UserAgentState.DISCONNECTED;
		lbState.setText(reason);
		btConnect.setText(CONNECT_TXT);
	}

	
}